
#include <stdint.h>      // define  int32_t  Ȃ
#include "stm32f30x.h"
#include "ICS2_STM32F30x.h"

static void USARTx_Config(uint32_t port, uint8_t preempt_level, uint8_t sub_level);

extern DMA_InitTypeDef      ICS_DMA_InitStructure;
extern DMA_Channel_TypeDef *ICS_USARTx_TX_DMA_CHANNEL;
extern uint32_t             ICS_USARTx_RDR_ADDRESS;
extern USART_TypeDef       *ICS_USARTx;
extern uint32_t             ICS_USARTx_speed;

extern uint32_t ics_waddr[SCOPE_SUPPORT_CHMAX];
extern int16_t  ics_wready;            /* 0:No data in ics_wbuf[] */
extern uint8_t  ics_rx_cnt;
extern uint8_t  ics_port_h4;
extern uint8_t  ics_tmode;
extern union
{
    uint8_t   txbuf[ 20];   /* for DMA buffer */
    uint16_t  txbufw[10];   /* 16bit          */
    uint32_t  txbufl[ 5];   /* 32bit          */
} ics_tx;



void ics2_init(uint32_t port, uint8_t preempt_level, uint8_t sub_level, uint32_t speed, uint8_t mode)
{
    int16_t  i;

  //ics_port     = port;
    ics_port_h4  = port>>12;
    ics_tmode    = mode;
    ICS_USARTx_speed = speed;

    for(i=0;i<SCOPE_SUPPORT_CHMAX;i++)
    {
        ics_waddr[i] = RAMSTARTADDR;  /* RAM START ADDR */
      //ics_wtype[i] = WTYPE_INT32;   /* 32bit int      */
    }

    /* USART configuration -----------------------------------------------------*/
    USARTx_Config(port, preempt_level, sub_level);

    ICS_DMA_InitStructure.DMA_BufferSize     = (uint16_t)18;
    ICS_DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)(&ics_tx.txbuf[0]);
    ICS_DMA_InitStructure.DMA_DIR            = DMA_DIR_PeripheralDST;
    ICS_DMA_InitStructure.DMA_Priority       = DMA_Priority_High;


    switch (port&ICS_USART_MASK)
    {
      case ICS_USART1:  // APB2
          ICS_DMA_InitStructure.DMA_PeripheralBaseAddr = 0x40013828;
          ICS_USARTx_RDR_ADDRESS                       = 0x40013824;
          break;
      case ICS_USART2:  // APB1
          ICS_DMA_InitStructure.DMA_PeripheralBaseAddr = 0x40004428;
          ICS_USARTx_RDR_ADDRESS                       = 0x40004424;
          break;
      case ICS_USART3:  // APB1
          ICS_DMA_InitStructure.DMA_PeripheralBaseAddr = 0x40004828;
          ICS_USARTx_RDR_ADDRESS                       = 0x40004824;
          break;
      case ICS_UART4:   // APB1
          ICS_DMA_InitStructure.DMA_PeripheralBaseAddr = 0x40004C28;
          ICS_USARTx_RDR_ADDRESS                       = 0x40004C24;
          break;
      default:
          break;
    }

    // DMA setting
    switch (port&ICS_DMA_MASK)
    {
      case ICS_DMA1CH2:
          ICS_USARTx_TX_DMA_CHANNEL = DMA1_Channel2;
          break;
      case ICS_DMA1CH4:
          ICS_USARTx_TX_DMA_CHANNEL = DMA1_Channel4;
          break;
      case ICS_DMA1CH7:
          ICS_USARTx_TX_DMA_CHANNEL = DMA1_Channel7;
          break;
      case ICS_DMA2CH5:
          ICS_USARTx_TX_DMA_CHANNEL = DMA2_Channel5;
          break;
      default:
          break;
    }


    DMA_DeInit(ICS_USARTx_TX_DMA_CHANNEL);
    DMA_Init(ICS_USARTx_TX_DMA_CHANNEL, &ICS_DMA_InitStructure);

    /* Enable the ICS_USARTx Receive interrupt:                 */
    /* This interrupt is generated when the ICS_USARTx receive  */
    /* data register is not empty                           */
    USART_ITConfig(ICS_USARTx, USART_IT_RXNE, ENABLE);
  //USART_ITConfig(ICS_USARTx, USART_IT_ERR,  ENABLE);
  //USART_ITConfig(ICS_USARTx, USART_IT_RXNE | USART_IT_ERR, ENABLE);

    ics_rx_cnt = 0;
    ics_wready = 0;   /* no data in ics_wbuf[] */
}


/*****************************************************
 @brief  Configures the USART Peripheral.
*****************************************************/
static void USARTx_Config(uint32_t port, uint8_t preempt_level, uint8_t sub_level)
{
    USART_InitTypeDef USART_InitStructure;
    NVIC_InitTypeDef  NVIC_InitStructure;
    GPIO_InitTypeDef  GPIO_InitStructure;

    uint16_t          USARTx_TX_PIN;
    GPIO_TypeDef     *USARTx_TX_GPIO_PORT;
  //uint32_t          USARTx_TX_GPIO_CLK;
    uint8_t           USARTx_TX_SOURCE;
    uint8_t           USARTx_TX_AF;
    uint16_t          USARTx_RX_PIN;
    GPIO_TypeDef     *USARTx_RX_GPIO_PORT;
  //uint32_t          USARTx_RX_GPIO_CLK;
    uint8_t           USARTx_RX_SOURCE;
    uint8_t           USARTx_RX_AF;
    uint8_t           USARTx_IRQn;


    switch (port)
    {
      case ICS_USART1_PB6_PB7_DMA1CH4:
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);    /* Enable GPIO clock  */
          RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE); /* Enable USART clock */
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);     /* Enable DMA clock   */

          USARTx_IRQn         = USART1_IRQn;
          ICS_USARTx          = USART1;

          USARTx_TX_PIN       = GPIO_Pin_6;
          USARTx_TX_GPIO_PORT = GPIOB;
          USARTx_TX_SOURCE    = GPIO_PinSource6;
          USARTx_TX_AF        = GPIO_AF_7;
          
          USARTx_RX_PIN       = GPIO_Pin_7;
          USARTx_RX_GPIO_PORT = GPIOB;
          USARTx_RX_SOURCE    = GPIO_PinSource7;
          USARTx_RX_AF        = GPIO_AF_7;
          break;

      case ICS_USART1_PC4_PC5_DMA1CH4:
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);    /* Enable GPIO clock  */
          RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE); /* Enable USART clock */
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);     /* Enable DMA clock   */

          USARTx_IRQn         = USART1_IRQn;
          ICS_USARTx          = USART1;

          USARTx_TX_PIN       = GPIO_Pin_4;
          USARTx_TX_GPIO_PORT = GPIOC;
          USARTx_TX_SOURCE    = GPIO_PinSource4;
          USARTx_TX_AF        = GPIO_AF_7;
          
          USARTx_RX_PIN       = GPIO_Pin_5;
          USARTx_RX_GPIO_PORT = GPIOC;
          USARTx_RX_SOURCE    = GPIO_PinSource5;
          USARTx_RX_AF        = GPIO_AF_7;
          break;

      case ICS_USART1_PA9_PA10_DMA1CH4:
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);    /* Enable GPIO clock  */
          RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE); /* Enable USART clock */
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);     /* Enable DMA clock   */

          USARTx_IRQn         = USART1_IRQn;
          ICS_USARTx          = USART1;

          USARTx_TX_PIN       = GPIO_Pin_9;
          USARTx_TX_GPIO_PORT = GPIOA;
          USARTx_TX_SOURCE    = GPIO_PinSource9;
          USARTx_TX_AF        = GPIO_AF_7;
          
          USARTx_RX_PIN       = GPIO_Pin_10;
          USARTx_RX_GPIO_PORT = GPIOA;
          USARTx_RX_SOURCE    = GPIO_PinSource10;
          USARTx_RX_AF        = GPIO_AF_7;
          break;

      case ICS_USART2_PB3_PB4_DMA1CH7:
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);    /* Enable GPIO clock  */
          RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE); /* Enable USART clock */
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);     /* Enable DMA clock   */

          USARTx_IRQn         = USART2_IRQn;
          ICS_USARTx          = USART2;

          USARTx_TX_PIN       = GPIO_Pin_3;
          USARTx_TX_GPIO_PORT = GPIOB;
          USARTx_TX_SOURCE    = GPIO_PinSource3;
          USARTx_TX_AF        = GPIO_AF_7;
          
          USARTx_RX_PIN       = GPIO_Pin_4;
          USARTx_RX_GPIO_PORT = GPIOB;
          USARTx_RX_SOURCE    = GPIO_PinSource4;
          USARTx_RX_AF        = GPIO_AF_7;
          break;

      case ICS_USART2_PA2_PA3_DMA1CH7:
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);    /* Enable GPIO clock  */
          RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE); /* Enable USART clock */
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);     /* Enable DMA clock   */

          USARTx_IRQn         = USART2_IRQn;
          ICS_USARTx          = USART2;

          USARTx_TX_PIN       = GPIO_Pin_2;
          USARTx_TX_GPIO_PORT = GPIOA;
          USARTx_TX_SOURCE    = GPIO_PinSource2;
          USARTx_TX_AF        = GPIO_AF_7;
          
          USARTx_RX_PIN       = GPIO_Pin_3;
          USARTx_RX_GPIO_PORT = GPIOA;
          USARTx_RX_SOURCE    = GPIO_PinSource3;
          USARTx_RX_AF        = GPIO_AF_7;
          break;

      case ICS_USART2_PA14_PA15_DMA1CH7:
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);    /* Enable GPIO clock  */
          RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE); /* Enable USART clock */
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);     /* Enable DMA clock   */

          USARTx_IRQn         = USART2_IRQn;
          ICS_USARTx          = USART2;

          USARTx_TX_PIN       = GPIO_Pin_14;
          USARTx_TX_GPIO_PORT = GPIOA;
          USARTx_TX_SOURCE    = GPIO_PinSource14;
          USARTx_TX_AF        = GPIO_AF_7;
          
          USARTx_RX_PIN       = GPIO_Pin_15;
          USARTx_RX_GPIO_PORT = GPIOA;
          USARTx_RX_SOURCE    = GPIO_PinSource15;
          USARTx_RX_AF        = GPIO_AF_7;
          break;

      case ICS_USART3_PC10_PC11_DMA1CH2:
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);    /* Enable GPIO clock  */
          RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE); /* Enable USART clock */
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);     /* Enable DMA clock   */

          USARTx_IRQn         = USART3_IRQn;
          ICS_USARTx          = USART3;

          USARTx_TX_PIN       = GPIO_Pin_10;
          USARTx_TX_GPIO_PORT = GPIOC;
          USARTx_TX_SOURCE    = GPIO_PinSource10;
          USARTx_TX_AF        = GPIO_AF_7;
          
          USARTx_RX_PIN       = GPIO_Pin_11;
          USARTx_RX_GPIO_PORT = GPIOC;
          USARTx_RX_SOURCE    = GPIO_PinSource11;
          USARTx_RX_AF        = GPIO_AF_7;
          break;

      case ICS_USART3_PB9_PB8_DMA1CH2:
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);    /* Enable GPIO clock  */
          RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE); /* Enable USART clock */
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);     /* Enable DMA clock   */

          USARTx_IRQn         = USART3_IRQn;
          ICS_USARTx          = USART3;

          USARTx_TX_PIN       = GPIO_Pin_9;
          USARTx_TX_GPIO_PORT = GPIOB;
          USARTx_TX_SOURCE    = GPIO_PinSource9;
          USARTx_TX_AF        = GPIO_AF_7;
          
          USARTx_RX_PIN       = GPIO_Pin_8;
          USARTx_RX_GPIO_PORT = GPIOB;
          USARTx_RX_SOURCE    = GPIO_PinSource8;
          USARTx_RX_AF        = GPIO_AF_7;
          break;

      case ICS_USART3_PB10_PB11_DMA1CH2:
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);    /* Enable GPIO clock  */
          RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE); /* Enable USART clock */
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);     /* Enable DMA clock   */

          USARTx_IRQn         = USART3_IRQn;
          ICS_USARTx          = USART3;

          USARTx_TX_PIN       = GPIO_Pin_10;
          USARTx_TX_GPIO_PORT = GPIOB;
          USARTx_TX_SOURCE    = GPIO_PinSource10;
          USARTx_TX_AF        = GPIO_AF_7;
          
          USARTx_RX_PIN       = GPIO_Pin_11;
          USARTx_RX_GPIO_PORT = GPIOB;
          USARTx_RX_SOURCE    = GPIO_PinSource11;
          USARTx_RX_AF        = GPIO_AF_7;
          break;
          
#if (STM32F303xE|STM32F303xC)          
      case ICS_UART4_PC10_PC11_DMA2CH5:
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);    /* Enable GPIO clock  */
          RCC_APB1PeriphClockCmd(RCC_APB1Periph_UART4, ENABLE);  /* Enable USART clock */
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA2, ENABLE);     /* Enable DMA clock   */

          USARTx_IRQn         = UART4_IRQn;
          ICS_USARTx          = UART4;

          USARTx_TX_PIN       = GPIO_Pin_10;
          USARTx_TX_GPIO_PORT = GPIOC;
          USARTx_TX_SOURCE    = GPIO_PinSource10;
          USARTx_TX_AF        = GPIO_AF_5;
          
          USARTx_RX_PIN       = GPIO_Pin_11;
          USARTx_RX_GPIO_PORT = GPIOC;
          USARTx_RX_SOURCE    = GPIO_PinSource11;
          USARTx_RX_AF        = GPIO_AF_5;
          break;
#endif
          
      default:
          break;
    }


    /* Connect PXx to USARTx_Tx */
    GPIO_PinAFConfig(USARTx_TX_GPIO_PORT, USARTx_TX_SOURCE, USARTx_TX_AF);

    /* Connect PXx to USARTx_Rx */
    GPIO_PinAFConfig(USARTx_RX_GPIO_PORT, USARTx_RX_SOURCE, USARTx_RX_AF);

    /* Configure USART Tx and Rx as alternate function push-pull */
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP;

    GPIO_InitStructure.GPIO_Pin   = USARTx_TX_PIN;
    GPIO_Init(USARTx_TX_GPIO_PORT, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin   = USARTx_RX_PIN;
    GPIO_Init(USARTx_RX_GPIO_PORT, &GPIO_InitStructure);

    /* ICS_USARTx configuration ----------------------------------------------------*/
    /* ICS_USARTx configured as follow:
    - BaudRate = defined by user
    - Word Length = 8 Bits
    - one Stop Bit
    - No parity
    - OverSampling 8times
    - Hardware flow control disabled (RTS and CTS signals)
    - Receive and transmit enabled
    */
    USART_InitStructure.USART_BaudRate   = ICS_USARTx_speed>>1;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits   = USART_StopBits_1;
    USART_InitStructure.USART_Parity     = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode       = USART_Mode_Rx | USART_Mode_Tx;
    USART_Init(ICS_USARTx, &USART_InitStructure);
    USART_OverSampling8Cmd(ICS_USARTx, ENABLE);

    /* DMA Configuration -------------------------------------------------------*/
    ICS_DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
    ICS_DMA_InitStructure.DMA_MemoryDataSize     = DMA_MemoryDataSize_Byte;
    ICS_DMA_InitStructure.DMA_PeripheralInc      = DMA_PeripheralInc_Disable;
    ICS_DMA_InitStructure.DMA_MemoryInc          = DMA_MemoryInc_Enable;
    ICS_DMA_InitStructure.DMA_Mode               = DMA_Mode_Normal;
    ICS_DMA_InitStructure.DMA_M2M                = DMA_M2M_Disable;

    /* NVIC configuration */
    /* Configure the Priority Group to 2 bits */
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);

    /* Enable the ICS_USARTx Interrupt */
    NVIC_InitStructure.NVIC_IRQChannel = USARTx_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = preempt_level;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = sub_level;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    /* Enable USART */
    USART_Cmd(ICS_USARTx, ENABLE);
}



