/**
  ******************************************************************************
  * File    ICS sample file for STM32F30x series
  * Version V1.0.0
  * Date    6-July-2017
  ******************************************************************************
  * Attention
  *
  * COPYRIGHT 2017 IAR systems
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "ICS2_STM32F30x.h"

/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
#define   PREEMPT     8   // Interrupt PREEMPTION PRIORITY
#define   SUB         0   // Interrupt SUB PRIORITY

/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
static void SysTickConfig(void);

/* Private functions ---------------------------------------------------------*/
void SystemInit(void);

int main(void)
{
    //************* HSI + PLL ******************** 
  
    //********************************************
    // HSI on ************************************
    do
    {
        RCC->CR |= 1;
    }
    while ( ((RCC->CR)&3)!=3 );  
  
    //********************************************
    // Select HSI clock as a system clock ********
    do 
    {
        RCC->CFGR = ((RCC->CFGR)&0xFFFFFFFC) | 0;
    }
    while (((RCC->CFGR)&0x0F)!=0x00 );    
  
    //********************************************
    // Stop PLL **********************************
    do
    {
        RCC->CR &= 0xFEFFFFFF;
    }
    while ( ((RCC->CR)&0x03000000)!=0x00000000);    
  
    //********************************************
    // Select HSI/2 as PLLinput ***********
    RCC->CFGR &= 0xFFFE7FFF;

    //********************************************
    // Set PLLMUL = x16 (value=1110b)=(64MHz)*****
    do
    {
        RCC->CFGR = ((RCC->CFGR) & 0xFFC3FFFF) | (14<<18);      
    }
    while ( (((RCC->CFGR)>>18)&0x0F)!=14 );    
  
    //********************************************
    // PLL on ************************************
    do
    {
        RCC->CR |= 0x01000000;
    }
    while ( ((RCC->CR)&0x03000000)!=0x03000000);  

    //********************************************
    // Select PLL output as a system clock *******
    do 
    {
        RCC->CFGR = ((RCC->CFGR)&0xFFFFFFFC) | 2;
    }
    while (((RCC->CFGR)&0x0F)!=0x0A );


    /* SysTick configuration ---------------------------------------------------*/
    SysTickConfig();


    // INT clock (HSI/2*16 = 64MHz) *********************************************   
#if  (STM32F303xE)

//*********** Tested on STM32F303RE
    ics2_init(ICS_USART1_PB6_PB7_DMA1CH4_F303xE,   PREEMPT, SUB, 4000000, 1); // OK CN9
//  ics2_init(ICS_USART1_PB6_PB7_DMA1CH4_F303xE,   PREEMPT, SUB, 4000000, 2); // OK CN9
//  ics2_init(ICS_USART1_PB6_PB7_DMA1CH4_F303xE,   PREEMPT, SUB, 4000000, 3); // OK CN9
//  ics2_init(ICS_USART1_PC4_PC5_DMA1CH4_F303xE,   PREEMPT, SUB, 4000000, 1); // OK CN4  
//  ics2_init(ICS_USART1_PA9_PA10_DMA1CH4_F303xE,  PREEMPT, SUB, 4000000, 1); // OK      nucleo
//  ics2_init(ICS_USART2_PB3_PB4_DMA1CH7_F303xE,   PREEMPT, SUB, 4000000, 1); // OK CN13 
//  ics2_init(ICS_USART2_PA2_PA3_DMA1CH7_F303xE,   PREEMPT, SUB, 4000000, 1); // OK CN1  
//  ics2_init(ICS_USART2_PA14_PA15_DMA1CH7_F303xE, PREEMPT, SUB, 4000000, 1); // OK CN11 j
//  ics2_init(ICS_USART3_PB9_PB8_DMA1CH2_F303xE,   PREEMPT, SUB, 4000000, 1); // OK CN12 r
//  ics2_init(ICS_USART3_PC10_PC11_DMA1CH2_F303xE, PREEMPT, SUB, 4000000, 1); // OK CN14 
//  ics2_init(ICS_USART3_PB10_PB11_DMA1CH2_F303xE, PREEMPT, SUB, 4000000, 1); // OK CN3  
//  ics2_init(ICS_UART4_PC10_PC11_DMA2CH5_F303xE,  PREEMPT, SUB, 4000000, 1); // OK CN14 

//*********** Tested on STM32F302RE
//  ics2_init(ICS_USART1_PB6_PB7_DMA1CH4_F302xE,   PREEMPT, SUB, 4000000, 1); // OK CN9
//  ics2_init(ICS_USART1_PC4_PC5_DMA1CH4_F302xE,   PREEMPT, SUB, 4000000, 1); // OK CN4  
//  ics2_init(ICS_USART1_PA9_PA10_DMA1CH4_F302xE,  PREEMPT, SUB, 4000000, 1); //         
//  ics2_init(ICS_USART2_PB3_PB4_DMA1CH7_F302xE,   PREEMPT, SUB, 4000000, 1); // OK CN13 
//  ics2_init(ICS_USART2_PA2_PA3_DMA1CH7_F302xE,   PREEMPT, SUB, 4000000, 1); // OK CN1  
//  ics2_init(ICS_USART2_PA14_PA15_DMA1CH7_F302xE, PREEMPT, SUB, 4000000, 1); //    CN11 j
//  ics2_init(ICS_USART3_PB9_PB8_DMA1CH2_F302xE,   PREEMPT, SUB, 4000000, 1); // OK CN12 r
//  ics2_init(ICS_USART3_PC10_PC11_DMA1CH2_F302xE, PREEMPT, SUB, 4000000, 1); // OK CN14 
//  ics2_init(ICS_USART3_PB10_PB11_DMA1CH2_F302xE, PREEMPT, SUB, 4000000, 1); // OK CN3  
//  ics2_init(ICS_UART4_PC10_PC11_DMA2CH5_F302xE,  PREEMPT, SUB, 4000000, 1); // OK CN14 

#elif (STM32F303xC)

//*********** Tested on STM32F303RC
    ics2_init(ICS_USART1_PB6_PB7_DMA1CH4_F303xC,   PREEMPT, SUB, 4000000, 1); // OK CN9
//  ics2_init(ICS_USART1_PC4_PC5_DMA1CH4_F303xC,   PREEMPT, SUB, 4000000, 1); // OK CN4
//  ics2_init(ICS_USART1_PA9_PA10_DMA1CH4_F303xC,  PREEMPT, SUB, 4000000, 1); //         
//  ics2_init(ICS_USART2_PB3_PB4_DMA1CH7_F303xC,   PREEMPT, SUB, 4000000, 1); // OK CN13
//  ics2_init(ICS_USART2_PA2_PA3_DMA1CH7_F303xC,   PREEMPT, SUB, 4000000, 1); // OK CN1 
//  ics2_init(ICS_USART2_PA14_PA15_DMA1CH7_F303xC, PREEMPT, SUB, 4000000, 1); //    CN11 j
//  ics2_init(ICS_USART3_PC10_PC11_DMA1CH2_F303xC, PREEMPT, SUB, 4000000, 1); // OK CN14
//  ics2_init(ICS_USART3_PB10_PB11_DMA1CH2_F303xC, PREEMPT, SUB, 4000000, 1); // OK CN3 
//  ics2_init(ICS_UART4_PC10_PC11_DMA2CH5_F303xC,  PREEMPT, SUB, 4000000, 1); // OK CN14

//*********** Tested on STM32F302RC
//  ics2_init(ICS_USART1_PB6_PB7_DMA1CH4_F302xC,   PREEMPT, SUB, 4000000, 1); // OK CN9
//  ics2_init(ICS_USART1_PC4_PC5_DMA1CH4_F302xC,   PREEMPT, SUB, 4000000, 1); // OK CN4
//  ics2_init(ICS_USART1_PA9_PA10_DMA1CH4_F302xC,  PREEMPT, SUB, 4000000, 1); //         
//  ics2_init(ICS_USART2_PB3_PB4_DMA1CH7_F302xC,   PREEMPT, SUB, 4000000, 1); // OK CN13
//  ics2_init(ICS_USART2_PA2_PA3_DMA1CH7_F302xC,   PREEMPT, SUB, 4000000, 1); // OK CN1 
//  ics2_init(ICS_USART2_PA14_PA15_DMA1CH7_F302xC, PREEMPT, SUB, 4000000, 1); //    CN11 j
//  ics2_init(ICS_USART3_PC10_PC11_DMA1CH2_F302xC, PREEMPT, SUB, 4000000, 1); // OK CN14
//  ics2_init(ICS_USART3_PB10_PB11_DMA1CH2_F302xC, PREEMPT, SUB, 4000000, 1); // OK CN3 
    ics2_init(ICS_UART4_PC10_PC11_DMA2CH5_F302xC,  PREEMPT, SUB, 4000000, 1); // OK CN14

#elif (STM32F334x8)
    
//*********** Tested on STM32F303R6,         
    ics2_init(ICS_USART1_PB6_PB7_DMA1CH4_F303x6,   PREEMPT, SUB, 4000000, 1); // OK CN9
//  ics2_init(ICS_USART1_PC4_PC5_DMA1CH4_F303x6,   PREEMPT, SUB, 4000000, 1); // OK CN4 
//  ics2_init(ICS_USART1_PA9_PA10_DMA1CH4_F303x6,  PREEMPT, SUB, 4000000, 1); //         
//  ics2_init(ICS_USART2_PB3_PB4_DMA1CH7_F303x6,   PREEMPT, SUB, 4000000, 1); // OK CN13
//  ics2_init(ICS_USART2_PA2_PA3_DMA1CH7_F303x6,   PREEMPT, SUB, 4000000, 1); // OK CN1  
//  ics2_init(ICS_USART2_PA14_PA15_DMA1CH7_F303x6, PREEMPT, SUB, 4000000, 1); //    CN11 j
//  ics2_init(ICS_USART3_PB9_PB8_DMA1CH2_F303x6,   PREEMPT, SUB, 4000000, 1); // OK CN12 r
//  ics2_init(ICS_USART3_PC10_PC11_DMA1CH2_F303x6, PREEMPT, SUB, 4000000, 1); // OK CN14
//  ics2_init(ICS_USART3_PB10_PB11_DMA1CH2_F303x6, PREEMPT, SUB, 4000000, 1); // OK CN3 

#elif (STM32F302x8)

//*********** Tested on STM32F302R8
    ics2_init(ICS_USART1_PB6_PB7_DMA1CH4_F302x8,   PREEMPT, SUB, 4000000, 1); // OK CN9 
//  ics2_init(ICS_USART1_PC4_PC5_DMA1CH4_F302x8,   PREEMPT, SUB, 4000000, 1); // OK CN4 
//  ics2_init(ICS_USART1_PA9_PA10_DMA1CH4_F302x8,  PREEMPT, SUB, 4000000, 1); //         
//  ics2_init(ICS_USART2_PB3_PB4_DMA1CH7_F302x8,   PREEMPT, SUB, 4000000, 1); // OK CN13 
//  ics2_init(ICS_USART2_PA2_PA3_DMA1CH7_F302x8,   PREEMPT, SUB, 4000000, 1); // OK CN1  
//  ics2_init(ICS_USART2_PA14_PA15_DMA1CH7_F302x8, PREEMPT, SUB, 4000000, 1); //    CN11 j
//  ics2_init(ICS_USART3_PB9_PB8_DMA1CH2_F302x8,   PREEMPT, SUB, 4000000, 1); // OK CN12 r
//  ics2_init(ICS_USART3_PC10_PC11_DMA1CH2_F302x8, PREEMPT, SUB, 4000000, 1); // OK CN14 
//  ics2_init(ICS_USART3_PB10_PB11_DMA1CH2_F302x8, PREEMPT, SUB, 4000000, 1); // OK CN3  
    
//*********** Tested on STM32F301R6    
//  ics2_init(ICS_USART1_PB6_PB7_DMA1CH4_F301x6,   PREEMPT, SUB, 4000000, 1); // OK CN9  
//  ics2_init(ICS_USART1_PC4_PC5_DMA1CH4_F301x6,   PREEMPT, SUB, 4000000, 1); // OK CN4  
//  ics2_init(ICS_USART1_PA9_PA10_DMA1CH4_F301x6,  PREEMPT, SUB, 4000000, 1); //         
//  ics2_init(ICS_USART2_PB3_PB4_DMA1CH7_F301x6,   PREEMPT, SUB, 4000000, 1); // OK CN13 
//  ics2_init(ICS_USART2_PA2_PA3_DMA1CH7_F301x6,   PREEMPT, SUB, 4000000, 1); // OK CN1  
//  ics2_init(ICS_USART2_PA14_PA15_DMA1CH7_F301x6, PREEMPT, SUB, 4000000, 1); //    CN11 j
//  ics2_init(ICS_USART3_PB9_PB8_DMA1CH2_F301x6,   PREEMPT, SUB, 4000000, 1); // OK CN12 r
//  ics2_init(ICS_USART3_PC10_PC11_DMA1CH2_F301x6, PREEMPT, SUB, 4000000, 1); // OK CN14 
//  ics2_init(ICS_USART3_PB10_PB11_DMA1CH2_F301x6, PREEMPT, SUB, 4000000, 1); // OK CN3  

#endif       
    
    while (1)
    {
      /* no code */
    }
}


static void SysTickConfig(void)
{
    /* Set SysTick Timer for 10000Hz = 100us interrupts  */
    if (SysTick_Config(SystemCoreClock / 10000))
    {
        /* Capture error */
        while (1);
    }
    /* Configure the SysTick handler priority */
    NVIC_SetPriority(SysTick_IRQn, 0x0);
}

/************************************************** END OF FILE****/
